use crate::{model, tasks::Task, ConfigFile, RunnerData};
use anyhow::{bail, Result};
use serde::Deserialize;
use serde_json::json;
use serde_repr::{Deserialize_repr, Serialize_repr};
use std::sync::Arc;

#[derive(Clone, Copy, Deserialize_repr, Serialize_repr, Debug)]
#[repr(i8)]
pub enum DefinitionStatus {
    CanceldTraining = -4,
    FailedTraining = -3,
    PreInit = 1,
    Init = 2,
    Training = 3,
    PausedTraining = 6,
    Tranied = 4,
    Ready = 5,
}

#[derive(Deserialize, Debug)]
pub struct Definition {
    pub id: String,
    pub model_id: String,
    pub accuracy: f64,
    pub target_accuracy: i64,
    pub epoch: i64,
    pub status: i64,
    pub created: String,
    pub epoch_progress: i64,
}

impl Definition {
    pub async fn updateStatus(
        self: &mut Definition,
        task: &Task,
        config: Arc<ConfigFile>,
        runner_data: Arc<RunnerData>,
        status: DefinitionStatus,
    ) -> Result<()> {
        println!("Marking Task as faield");

        let client = reqwest::Client::new();

        let to_send = json!({
            "id": runner_data.id,
            "taskId": task.id,
            "defId": self.id,
            "status": status,
        });

        let resp = client
            .post(format!("{}/tasks/runner/train/def/status", config.hostname))
            .header("token", &config.token)
            .body(to_send.to_string())
            .send()
            .await?;

        if resp.status() != 200 {
            println!("Failed to update status of task");
            bail!("Failed to update status of task");
        }

        Ok(())
    }
}

#[derive(Clone, Copy, Deserialize_repr, Debug)]
#[repr(i8)]
pub enum ModelClassStatus {
    ToTrain = 1,
    Training = 2,
    Trained = 3,
}

#[derive(Deserialize, Debug)]
pub struct ModelClass {
    pub id: String,
    pub model_id: String,
    pub name: String,
    pub class_order: i64,
    pub status: ModelClassStatus,
}

#[derive(Deserialize, Debug)]
pub struct DataPointRequest {
    pub testing: Vec<model::DataPoint>,
    pub training: Vec<model::DataPoint>,
}
